@@ -82,7 +82,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.5 2001/07/12 19:59:30 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.3 2002/08/21 21:01:55 shadow Exp $");
 
 #include <stdio.h> 
 #include <stdlib.h>      /* for malloc() */
@@ -358,7 +358,12 @@ static TAdd(cb, thead)
 
 	cb->tprev = thp->tprev;
 	cb->tnext = *thead;
-	thp->tprev = (itocb(thp->tprev)->tnext = cbtoi(cb));
+	if (thp) {
+	  if (thp->tprev) 
+	    thp->tprev = (itocb(thp->tprev)->tnext = cbtoi(cb));
+	  else 
+	    thp->tprev = cbtoi(cb);
+	}
     }
     cb->thead = ttoi(thead);
 
@@ -374,8 +379,10 @@ static TDel(cb)
 
     if (*thead == cbtoi(cb))
 	*thead = (*thead == cb->tnext? 0: cb->tnext);
-    itocb(cb->tprev)->tnext = cb->tnext;
-    itocb(cb->tnext)->tprev = cb->tprev;
+    if (itocb(cb->tprev)) 
+      itocb(cb->tprev)->tnext = cb->tnext;
+    if (itocb(cb->tnext)) 
+      itocb(cb->tnext)->tprev = cb->tprev;
 
 } /*TDel*/
 
@@ -1384,16 +1391,16 @@ static int lih_r(host, held, hostp)
 
 } /*lih*/
 
-
 /* This could be upgraded to get more space each time */
 /* first pass: find the oldest host which isn't held by anyone */
 /* second pass: find the oldest host who isn't "me" */
 /* always called with hostp unlocked */
+extern struct host *hostList;
 static int GetSomeSpace_r(hostp, locked)
     struct host *hostp;
     int locked;
 {
-    register struct host *hp, *hp1 = (struct host *)0;
+    register struct host *hp, *hp1 = (struct host *)0, *hp2 = hostList;
     int i=0;
 
     cbstuff.GotSomeSpaces++;
@@ -1404,13 +1411,15 @@ static int GetSomeSpace_r(hostp, locked)
     }
     do {
 	lih_host = 0;
-	h_Enumerate_r(lih_r, (char *)hp1);
+	h_Enumerate_r(lih_r, hp2, (char *)hp1);
 	hp = lih_host;
 	if (hp) {
 	    cbstuff.GSS4++;
 	    if ( ! ClearHostCallbacks_r(hp, 0 /* not locked or held */) )
 		return;
+	    hp2 = hp->next;
 	} else {
+	    hp2 = hostList;
 	    hp1 = hostp;
 	    cbstuff.GSS1++;
 	    ViceLog(5,("GSS: Try harder for longest inactive host cnt= %d\n", i));
@@ -1631,7 +1640,7 @@ main(argc, argv)
     register struct CallBack *cb;
     time_t now;
 
-    bzero(&fid, sizeof(fid));
+    memset(&fid, 0, sizeof(fid));
     argc--; argv++;
     while (argc && **argv == '-') {
 	noptions++;
